VERSION = (0, 1, 3, 'stable')


def get_release():
    return '-'.join([get_version(), VERSION[-1]])


def get_version():
    """
    Returns only digit parts of version.
    """
    return '.'.join(str(i) for i in VERSION[:3])


__author__ = 'dlancer'
__docformat__ = 'restructuredtext en'
__copyright__ = 'Copyright 2015-2016, dlancer'
__license__ = 'BSD'
__version__ = get_release()
__maintainer__ = 'dlancer'
__email__ = 'dmdpost@gmail.com'
__status__ = 'Development'



# -*- coding: utf-8
from __future__ import unicode_literals
from django.contrib.auth.models import AnonymousUser
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.utils import translation

from pages.cache import cache
from pages.models import PageSlugContent, PageMetaContent
from pages.models import PageTextContent
from pages.views import PageDetailsView
from pagesext.tests.base import PagesExtCase
from pagesext.models import PageTagsContent, PageImageContent, PageVideoContent, PageFileContent


class TestExtPages(PagesExtCase):
    def test_page_tags_model(self):
        PageTagsContent.objects.create(page=self.page_foo)
        obj = PageTagsContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:tags:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:tags:1')

    def test_page_tags_api(self):
        PageTagsContent.objects.create(page=self.page_foo)
        obj = PageTagsContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:tags:1')
        obj.add('tag1', 'tag2')
        names = sorted(list(obj.tags.names()))
        self.assertEqual(names[0], 'tag1')
        self.assertEqual(names[1], 'tag2')
        obj.set('tag3', 'tag4')
        names = sorted(list(obj.tags.names()))
        self.assertEqual(names[0], 'tag3')
        self.assertEqual(names[1], 'tag4')
        slugs = sorted(list(obj.tags.slugs()))
        self.assertEqual(slugs[0], 'tag3')
        self.assertEqual(slugs[1], 'tag4')
        PageTagsContent.objects.create(page=self.page_foo2)
        obj2 = PageTagsContent.objects.filter(page=self.page_foo2, language='en')[0]
        obj2.add('tag4', 'tag5')
        objects = obj.similar_objects()
        self.assertEqual(objects[0], obj2)
        obj.remove('tag3')
        names = obj.tags.names()
        self.assertEqual(len(names), 1)
        obj.clear()
        names = obj.tags.names()
        self.assertEqual(len(names), 0)

    def test_page_image_model(self):
        PageImageContent.objects.create(page=self.page_foo)
        obj = PageImageContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:image:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:image:1')

    def test_page_video_model(self):
        PageVideoContent.objects.create(page=self.page_foo)
        obj = PageVideoContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:video:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:video:1')

    def test_page_file_model(self):
        PageFileContent.objects.create(page=self.page_foo)
        obj = PageFileContent.objects.filter(page=self.page_foo, language='en')[0]
        sid = obj.sid
        self.assertEqual(sid, 'en:Test:file:1')
        obj.language = 'de'
        obj.save()
        self.assertEqual(obj.sid, 'de:Test:file:1')

    def test_page_tags_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageTagsContent.objects.create(page=self.page_foo)
        obj = PageTagsContent.objects.filter(page=self.page_foo, language='en')[0]
        obj.add('tag1', 'tag2')
        self.page_foo.template = 'pages/page_tags.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_image_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageImageContent.objects.create(page=self.page_foo, image='img/test.jpg', title='test')
        self.page_foo.template = 'pages/page_image.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_video_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageVideoContent.objects.create(page=self.page_foo, video='xxx', title='test')
        self.page_foo.template = 'pages/page_video.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()

    def test_page_file_view(self):
        PageSlugContent.objects.create(page=self.page_foo, slug='test')
        PageMetaContent.objects.create(page=self.page_foo, title='test', description='test', keywords='test')
        PageTextContent.objects.create(page=self.page_foo, text='test')
        PageFileContent.objects.create(page=self.page_foo, file='files/test.txt', title='test')
        self.page_foo.template = 'pages/page_file.html'
        self.page_foo.save()
        page_url = reverse('pages:show', kwargs={'slug': 'test'})
        request = self.factory.get(page_url)
        request.user = AnonymousUser()
        context = RequestContext(request)
        view = PageDetailsView.as_view()
        translation.activate('en')
        response = view(request=request, context=context, slug='test')
        translation.deactivate()
        self.assertEqual(response.status_code, 200)
        self.page_foo.delete()
        cache.clear()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import pagesext.storage
import pagesext.models.pageimagecontent
import embed_video.fields
import pagesext.models.pagefilecontent
import django.utils.timezone
from django.conf import settings
import image_cropping.fields
import taggit.managers


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0002_auto_20150616_2121'),
        ('pages', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PageFileContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, verbose_name='Type', db_index=True)),
                ('language', models.CharField(default=b'en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(default=False, verbose_name='Extended?')),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created')),
                ('date_updated', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Updated')),
                ('file', models.FileField(storage=pagesext.storage.PageExtFileSystemStorage(), null=True, upload_to=pagesext.models.pagefilecontent.make_file_upload_path, blank=True)),
                ('title', models.CharField(max_length=160, blank=True)),
                ('description', models.TextField(max_length=160, blank=True)),
                ('created_by', models.ForeignKey(related_name='pagesext_pagefilecontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags')),
                ('updated_by', models.ForeignKey(related_name='pagesext_pagefilecontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'File',
                'verbose_name_plural': 'File',
            },
        ),
        migrations.CreateModel(
            name='PageImageContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, verbose_name='Type', db_index=True)),
                ('language', models.CharField(default=b'en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(default=False, verbose_name='Extended?')),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created')),
                ('date_updated', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Updated')),
                ('image', image_cropping.fields.ImageCropField(storage=pagesext.storage.PageExtFileSystemStorage(), null=True, upload_to=pagesext.models.pageimagecontent.make_image_upload_path, blank=True)),
                ('cropping', image_cropping.fields.ImageRatioField('image', '600x800', hide_image_field=False, size_warning=True, allow_fullsize=True, free_crop=False, adapt_rotation=False, help_text=None, verbose_name='cropping')),
                ('title', models.CharField(max_length=250, blank=True)),
                ('created_by', models.ForeignKey(related_name='pagesext_pageimagecontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags')),
                ('updated_by', models.ForeignKey(related_name='pagesext_pageimagecontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Image',
                'verbose_name_plural': 'Images',
            },
        ),
        migrations.CreateModel(
            name='PageTagsContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, verbose_name='Type', db_index=True)),
                ('language', models.CharField(default=b'en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(default=False, verbose_name='Extended?')),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created')),
                ('date_updated', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Updated')),
                ('created_by', models.ForeignKey(related_name='pagesext_pagetagscontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags')),
                ('updated_by', models.ForeignKey(related_name='pagesext_pagetagscontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Tags',
                'verbose_name_plural': 'Tags',
            },
        ),
        migrations.CreateModel(
            name='PageVideoContent',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('type', models.CharField(max_length=100, verbose_name='Type', db_index=True)),
                ('language', models.CharField(default=b'en', max_length=5)),
                ('sid', models.CharField(unique=True, max_length=200)),
                ('name', models.CharField(unique=True, max_length=200, blank=True)),
                ('is_extended', models.BooleanField(default=False, verbose_name='Extended?')),
                ('comment', models.CharField(max_length=250, blank=True)),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Created')),
                ('date_updated', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Updated')),
                ('video', embed_video.fields.EmbedVideoField(blank=True)),
                ('title', models.CharField(max_length=160, blank=True)),
                ('description', models.TextField(max_length=160, blank=True)),
                ('created_by', models.ForeignKey(related_name='pagesext_pagevideocontent_creator', to=settings.AUTH_USER_MODEL, null=True)),
                ('page', models.ForeignKey(verbose_name='Page', to='pages.Page')),
                ('tags', taggit.managers.TaggableManager(to='taggit.Tag', through='taggit.TaggedItem', help_text='A comma-separated list of tags.', verbose_name='Tags')),
                ('updated_by', models.ForeignKey(related_name='pagesext_pagevideocontent_editor', to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Video',
                'verbose_name_plural': 'Video',
            },
        ),
    ]



"""Implements PageExtFileSystemStorage class """

import os

from django.core.files.storage import FileSystemStorage

from pagesext.conf import settings


class PageExtBaseFileSystemStorage(FileSystemStorage):
    def __log(self, log):
        if self.logger:
            self.logger.log(log)

    def __init__(self, location=None, logger=None):
        # Get location from Settings, if not mentioned PAGES_FILE_LOCATION must be set in settings
        if not location:
            location = settings.PAGES_EXT_FILE_LOCATION
        self.logger = logger
        # Initialize super class
        FileSystemStorage.__init__(self, location=location)
        self.file_permissions_mode = settings.PAGES_EXT_FILE_UPLOAD_PERMISSIONS
        if getattr(settings, 'FILE_UPLOAD_DIRECTORY_PERMISSIONS', False):
            self.directory_permissions_mode = settings.PAGES_EXT_FILE_UPLOAD_DIRECTORY_PERMISSIONS

        # add log
        self.__log('PageExtFileSystemStorage initialized.')

    # override default behavior where default mode is 'rb'
    def open(self, name, mode='r'):
        return FileSystemStorage.open(self, name, mode)

    def _open(self, name, mode='rb'):
        self.__log('Opening file ' + name + ' in mode=' + mode)
        return FileSystemStorage._open(self, name, mode)

    def exists(self, name):
        exist = os.path.exists(self.path(name))
        if exist:
            self.__log('File ' + name + ' exists.')
        else:
            self.__log('File ' + name + ' does not exist.')
        return exist

    def delete(self, name):
        FileSystemStorage.delete(self, name)
        self.__log('File ' + name + ' deleted.')


class PageExtFileSystemStorage(PageExtBaseFileSystemStorage):

    def get_available_name(self, name, max_length=None):
        # If the filename already exists, remove it as if it was a true file system
        if settings.PAGES_EXT_FILE_OVERWRITE_EXISTS:
            if self.exists(name):
                self.__log('File ' + name + ' exists, rewrite used.')
                os.remove(os.path.join(settings.MEDIA_ROOT, name))
        else:
            name = FileSystemStorage.get_available_name(self, name, max_length)
        return name



"""Implements admin interface for page tags content"""

from django.contrib import admin

from pages.conf import settings

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:
    from pagesext.models.pagetagscontent import PageTagsContent

    class PageTagsContentAdmin(admin.ModelAdmin):
        list_display = ('__str__', 'created_by', 'updated_by', 'date_created', 'date_updated',)
        list_display_links = ['__str__']
        fieldsets = [
            (None, {'fields': [
                ('language', ),
                ('name', ),
                ('tags', ),
                ('comment', ),
            ]}),
        ]

        exclude = ('type', 'sid', 'is_extended', 'created_by', 'updated_by',)
        save_on_top = True
        actions_on_bottom = True

    admin.site.register(PageTagsContent, PageTagsContentAdmin)

    class PageTagsContentInline(admin.StackedInline):
        model = PageTagsContent
        extra = 1
        exclude = ('sid', 'is_extended', 'created_by', 'updated_by', 'date_created', 'date_updated',)
        fieldsets = [
            (None, {'fields': [
                ('language', ),
                ('name', ),
                ('tags', ),
                ('comment', ),
            ]}),
        ]



"""Implements page file extended content model"""

from __future__ import unicode_literals

import os
import uuid
import hashlib
import base64

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.db.models.signals import post_delete
from django.dispatch import receiver

from taggit.managers import TaggableManager

from pages.conf import settings
from pages.models.pagebasecontent import PageBaseContent
from pagesext.storage import PageExtFileSystemStorage
from pagesext.conf import settings as ext_settings

if settings.PAGES_PAGE_USE_EXT_CONTENT_TYPES:

    file_storage = PageExtFileSystemStorage()

    def make_file_upload_path(instance, filename, prefix=False):
        """Generate upload path and filename for file"""

        # store original filename
        instance.filename = filename
        ext = os.path.splitext(filename)[1].strip('.').lower()

        # always generate unique path, for security and file system balancing reasons
        file_uuid = uuid.uuid4().hex

        # generate short sha1 based hash
        name_sha = hashlib.sha1(file_uuid).digest()
        name_hash = base64.urlsafe_b64encode(name_sha[:9]).decode('utf-8')

        if ext_settings.PAGES_EXT_FILE_USE_ORIGINAL_FILENAME:
            name = file_storage.get_available_name(os.path.splitext(filename)[0])
        else:
            name = name_hash

        return u'{path}/{sub0}/{sub1}/{sub2}/{name}.{ext}'.format(
            path=ext_settings.PAGES_EXT_FILE_DIR,
            sub0=file_uuid[0:2],
            sub1=file_uuid[7:9],
            sub2=file_uuid[12:14],
            name=name,
            ext=ext
        )

    class PageFileContent(PageBaseContent):
        file = models.FileField(blank=True, null=True, upload_to=make_file_upload_path, storage=file_storage)
        title = models.CharField(max_length=160, blank=True)
        description = models.TextField(max_length=160, blank=True)
        tags = TaggableManager()
        objects = models.Manager()

        def update_fields(self, change):
            if not change:
                self.type = 'file'
                self.is_extended = True

        class Meta(PageBaseContent.Meta):
            app_label = 'pagesext'
            verbose_name = _('File')
            verbose_name_plural = _('File')

        class PageMeta(PageBaseContent.PageMeta):
            context_name = 'file'
            multiple_per_page = True
            multiple_per_locale = True

    # Signals handler for deleting files after object record deleted
    @receiver(post_delete, sender=PageFileContent)
    def delete_file(sender, **kwargs):
        """Automatically delete file when records removed."""
        if ext_settings.PAGES_EXT_FILE_DELETE_FILE:
            obj = kwargs.get('instance')
            if obj.file is not None:
                try:
                    obj.file.storage.delete(obj.file.path)
                except Exception:
                    pass
